fn main() {
    let input = std::fs::read_to_string("input.txt").unwrap();

    let result = valid_game_id_sum(&input);

    println!("Result: {}", result);
}

const MAX_RED: u32 = 12;
const MAX_GREEN: u32 = 13;
const MAX_BLUE: u32 = 14;

fn valid_game_id_sum(game_list: &str) -> u32 {
    let res: u32 = game_list.lines().filter_map(|game| {
        let (left, right) = game.split_once(": ").unwrap();
        if game_validity_check(right.trim()) {
            let game_id: u32 = left.trim_start_matches("Game ").parse().unwrap();
            Some(game_id)
        }
        else {
            None
        }
    }).sum();

    res
}

fn game_validity_check(game: &str) -> bool {
    for set in game.split("; ") {
        let colors = set.split(", ");
        for color in colors {
            let (count, col) = color.split_once(' ').unwrap();
            let count: u32 = count.parse().unwrap();
            let invalid = match col {
                "red" => count > MAX_RED,
                "green" => count > MAX_GREEN,
                "blue" => count > MAX_BLUE,
                _ => panic!("invalid color"),
            };
            if invalid {
                return false;
            }
        }
    }

    true
}

#[cfg(test)]
mod test {
    use crate::valid_game_id_sum;

    const PART_1_EXAMPLE: &str = "Game 1: 3 blue, 4 red; 1 red, 2 green, 6 blue; 2 green
Game 2: 1 blue, 2 green; 3 green, 4 blue, 1 red; 1 green, 1 blue
Game 3: 8 green, 6 blue, 20 red; 5 blue, 4 red, 13 green; 5 green, 1 red
Game 4: 1 green, 3 red, 6 blue; 3 green, 6 red; 3 green, 15 blue, 14 red
Game 5: 6 red, 1 blue, 3 green; 2 blue, 1 red, 2 green";

    #[test]
    fn part_1_example() {
        assert_eq!(8, valid_game_id_sum(PART_1_EXAMPLE));
    }
}